using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NVPP;
using GASS.CUDA;
using GASS.CUDA.Types;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace NVPPTranspose
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            //original = (Bitmap)Bitmap.FromFile(@"res\fuchsia.jpg_bw.bmp");
            original = (Bitmap)Bitmap.FromFile(@"res\fuchsia.jpg_bw.bmp");
            pcbOrig.Image = original;
            dst = new Bitmap(original.Width, original.Height, PixelFormat.Format8bppIndexed);
        }

        private void btnTranspose_Click(object sender, EventArgs e)
        {
            int srcPitch = 0;
            int dstPitch = 0;
            CUResult cuda_res = CUResult.Success;

            CUdeviceptr orig = NVPPWrapper.Malloc_C1(original.Width, original.Height, ref srcPitch);
            CUdeviceptr dstBuf = NVPPWrapper.Malloc_C1(dst.Width, dst.Height, ref dstPitch);

            if (orig.Pointer == 0)
            {
                MessageBox.Show("Error allocating memory");
                return;
            }

            NvppStatus res = NVPPWrapper.Set_C1(27, orig, srcPitch, new NvppiSize(original.Width, original.Height));
            //MessageBox.Show(res.ToString());
            res = NVPPWrapper.Set_C1(27, dstBuf, dstPitch, new NvppiSize(dst.Width, dst.Height));

            //BitmapData origData = original.LockBits(Rectangle.FromLTRB(0, 0, original.Width, original.Height), ImageLockMode.ReadWrite, PixelFormat.Format1bppIndexed);
            // Convert 1bpp to 8bpp
            IntPtr tempOrig = Marshal.AllocHGlobal(srcPitch * original.Height);
            for (int i = 0; i < original.Height; i++)
            {
                for (int j = 0; j < original.Width; j++)
                {
                    //int offset = i * origData.Stride + j >> 3;
                    if (original.GetPixel(j, i).ToArgb() == Color.White.ToArgb())
                    {
                        Marshal.WriteByte(tempOrig, i * srcPitch + j, byte.MaxValue);
                    }
                    else
                    {
                        Marshal.WriteByte(tempOrig, i * srcPitch + j, byte.MinValue);
                    }
                }
            }
            CUDAMemCpy2D copy = new CUDAMemCpy2D();
            copy.srcMemoryType = CUMemoryType.Host;
            //copy.srcHost = origData.Scan0;
            copy.srcHost = tempOrig;
            copy.srcPitch = (uint)srcPitch;
            copy.dstMemoryType = CUMemoryType.Device;
            copy.dstDevice = orig;
            copy.dstPitch = (uint)srcPitch;
            copy.WidthInBytes = (uint)(original.Width);
            copy.Height = (uint)original.Height;
            cuda_res = CUDADriver.cuMemcpy2D(ref copy);
            //original.UnlockBits(origData);
            Marshal.FreeHGlobal(tempOrig);

            res = NVPPWrapper.Transpose(
                orig,
                srcPitch,
                dstBuf,
                dstPitch,
                new NvppiSize(dst.Width, dst.Height));

            BitmapData dstData = dst.LockBits(Rectangle.FromLTRB(0, 0, dst.Width, dst.Height), ImageLockMode.WriteOnly, PixelFormat.Format8bppIndexed);
            copy = new CUDAMemCpy2D();
            copy.srcMemoryType = CUMemoryType.Device;
            copy.srcDevice = dstBuf;
            copy.srcPitch = (uint)dstPitch;
            copy.dstMemoryType = CUMemoryType.Host;
            copy.dstHost = dstData.Scan0;
            copy.dstPitch = (uint)dstData.Stride;
            copy.WidthInBytes = (uint)(dst.Width);
            copy.Height = (uint)dst.Height;
            cuda_res = CUDADriver.cuMemcpy2D(ref copy);
            dst.UnlockBits(dstData);

            // Show result.
            disp.ClientSize = new Size(dst.Width, dst.Height);
            //disp.Size.Width = 1024;
            //disp.Size.Height = 768;
            disp.StartPosition = FormStartPosition.Manual;
            disp.Top = 0;
            disp.Left = 0;
            PictureBox pcb = new PictureBox();
            pcb.Size = disp.ClientSize;
            pcb.Image = dst;
            disp.Controls.Add(pcb);

            disp.Show();
        }

        Bitmap original;
        Bitmap dst;
        Form disp = new Form();
    }
}