using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using GASS.CUDA.Types;

namespace NVPP
{
    enum NvppStatus
    {
        // negative return-codes indicate errors
        NVPP_NOT_SUPPORTED_MODE_ERROR = -9999,
        NVPP_ROUND_MODE_NOT_SUPPORTED_ERROR = -213,
        NVPP_RESIZE_NO_OPERATION_ERROR = -50,
        NVPP_BAD_ARG_ERROR = -26,
        NVPP_LUT_NUMBER_OF_LEVELS_ERROR = -25,
        NVPP_TEXTURE_BIND_ERROR = -24,
        NVPP_COEFF_ERROR = -23,
        NVPP_RECT_ERROR = -22,
        NVPP_QUAD_ERROR = -21,
        NVPP_WRONG_INTERSECTION_ROI_ERROR = -20,
        NVPP_NOT_EVEN_STEP_ERROR = -19,
        NVPP_INTERPOLATION_ERROR = -18,
        NVPP_RESIZE_FACTOR_ERROR = -17,
        NVPP_HAAR_CLASSIFIER_PIXEL_MATCH_ERROR = -16,
        NVPP_MEMFREE_ERR = -15,
        NVPP_MEMSET_ERR = -14,
        NVPP_MEMCPY_ERROR = -13,
        NVPP_MEM_ALLOC_ERR = -12,
        NVPP_HISTO_NUMBER_OF_LEVELS_ERROR = -11,
        NVPP_MIRROR_FLIP_ERR = -10,
        NVPP_INVALID_INPUT = -9,
        NVPP_ALIGNMENT_ERROR = -8,
        NVPP_STEP_ERROR = -7,
        NVPP_SIZE_ERROR = -6,
        NVPP_POINTER_ERROR = -5,
        NVPP_NULL_POINTER_ERROR = -4,
        NVPP_CUDA_KERNEL_EXECUTION_ERROR = -3,
        NVPP_NOT_IMPLEMENTED_ERROR = -2,
        NVPP_ERROR = -1,

        // success
        NVPP_NO_ERROR = 0,
        NVPP_SUCCESS = NVPP_NO_ERROR,

        // positive return-codes indicate warnings
        NVPP_WARNING = 1,
        NVPP_WRONG_INTERSECTION_QUAD_WARNING = 2,
        NVPP_MISALIGNED_DST_ROI_WARNING = 3,
        NVPP_AFFINE_QUAD_INCORRECT_WARNING = 4,
        NVPP_ADJUSTED_ROI_SIZE_WARNING = 5,
        NVPP_ODD_ROI_WARNING = 6,
    }

    struct NvppiSize
    {
        public NvppiSize(int width, int height)
        {
            this.width = width;
            this.height = height;
        }

        public int width;
        public int height;
    }

    struct NvppiRect
    {
        public NvppiRect(int x, int y, int width, int height)
        {
            this.x = x;
            this.y = y;
            this.width = width;
            this.height = height;
        }

        public int x;
        public int y;
        public int width;
        public int height;
    }

    struct NvppiPoint
    {
        public NvppiPoint(int x, int y)
        {
            this.x = x;
            this.y = y;
        }

        public int x;
        public int y;
    }

    class NVPPWrapper
    {
        // This is an example of an exported function.
        [DllImport("CPPWrapper.dll")]
        public static extern CUdeviceptr Malloc_C1(int width, int height, ref int pitch);

        // This is an example of an exported function.
        [DllImport("CPPWrapper.dll")]
        public static extern void Free(CUdeviceptr buffer);

        [DllImport("CPPWrapper.dll")]
        public static extern CUdeviceptr Malloc_C4(int width, int height, ref int pitch);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Set_C4(int value, CUdeviceptr buffer, int pitch, NvppiSize size);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Set_C1(byte value, CUdeviceptr buffer, int pitch, NvppiSize size);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Resize(CUdeviceptr srcBuf, NvppiSize srcSize, int srcPitch, NvppiRect srcROI, CUdeviceptr dstBuf, int dstPitch, NvppiSize dstROISize, double xFactor, double yFactor, int interp);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Rotate(CUdeviceptr srcBuf, NvppiSize srcSize, int srcPitch, NvppiRect srcROI, CUdeviceptr dstBuf, int dstPitch, NvppiRect dstROI, double angle, double xShift, double yShift, int interp);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Erode(CUdeviceptr srcBuf, int srcPitch, CUdeviceptr dstBuf, int dstPitch, NvppiSize ROI, CUdeviceptr mask, NvppiSize maskSize, NvppiPoint anchor);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Erode(CUdeviceptr srcBuf, int srcPitch, CUdeviceptr dstBuf, int dstPitch, NvppiSize ROI, byte[] mask, NvppiSize maskSize, NvppiPoint anchor);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Transpose(CUdeviceptr srcBuf, int srcPitch, CUdeviceptr dstBuf, int dstPitch, NvppiSize ROI);

        [DllImport("CPPWrapper.dll")]
        public static extern NvppStatus Histogram(CUdeviceptr srcBuf, int srcPitch, NvppiSize ROI, CUdeviceptr histogram, CUdeviceptr levels, int nLevels, int lowerLevel, int upperLevel);
    }
}
