using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using GASS.CUDA.Types;
using NVPP;
using GASS.CUDA;
using System.Drawing.Imaging;

namespace NVPPResize
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            original = (Bitmap)Bitmap.FromFile(@"res\Helianthus_divaricatus_flowers.jpg.bmp");
            pcbOrig.Image = original;
            dst = new Bitmap(1024, 768, PixelFormat.Format32bppArgb);
        }

        private void btnResize_Click(object sender, EventArgs e)
        {
            int srcPitch = 0;
            int dstPitch = 0;

            CUdeviceptr orig = NVPPWrapper.Malloc_C4(original.Width, original.Height, ref srcPitch);
            CUdeviceptr dstBuf = NVPPWrapper.Malloc_C4(dst.Width, dst.Height, ref dstPitch);

            if (orig.Pointer == 0)
            {
                MessageBox.Show("Error allocating memory");
                return;
            }

            NvppStatus res = NVPPWrapper.Set_C4(Color.Yellow.ToArgb(), orig, srcPitch, new NvppiSize(original.Width, original.Height));
            //MessageBox.Show(res.ToString());
            res = NVPPWrapper.Set_C4(Color.Blue.ToArgb(), dstBuf, dstPitch, new NvppiSize(dst.Width, dst.Height));

            BitmapData origData = original.LockBits(Rectangle.FromLTRB(0, 0, original.Width, original.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            CUDAMemCpy2D copy = new CUDAMemCpy2D();
            copy.srcMemoryType = CUMemoryType.Host;
            copy.srcHost = origData.Scan0;
            copy.srcPitch = (uint)origData.Stride;
            copy.dstMemoryType = CUMemoryType.Device;
            copy.dstDevice = orig;
            copy.dstPitch = (uint)srcPitch;
            copy.WidthInBytes = (uint)(original.Width * 4);
            copy.Height = (uint)original.Height;
            CUResult cuda_res = CUDADriver.cuMemcpy2D(ref copy);
            original.UnlockBits(origData);

            res = NVPPWrapper.Resize(
                orig, 
                new NvppiSize(original.Width, original.Height), 
                srcPitch, 
                new NvppiRect(0, 0, original.Width, original.Height), 
                dstBuf, 
                dstPitch, 
                new NvppiSize(dst.Width, dst.Height), 
                ((double)dst.Width) / ((double)original.Width),
                ((double)dst.Height) / ((double)original.Height), 
                1);

            BitmapData dstData = dst.LockBits(Rectangle.FromLTRB(0, 0, dst.Width, dst.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            copy = new CUDAMemCpy2D();
            copy.srcMemoryType = CUMemoryType.Device;
            copy.srcDevice = dstBuf;
            copy.srcPitch = (uint)dstPitch;
            copy.dstMemoryType = CUMemoryType.Host;
            copy.dstHost = dstData.Scan0;
            copy.dstPitch = (uint)dstData.Stride;
            copy.WidthInBytes = (uint)(dst.Width * 4);
            copy.Height = (uint)dst.Height;
            cuda_res = CUDADriver.cuMemcpy2D(ref copy);
            dst.UnlockBits(dstData);

            // Show result.
            disp.ClientSize = new Size(dst.Width, dst.Height);
            //disp.Size.Width = 1024;
            //disp.Size.Height = 768;
            disp.StartPosition = FormStartPosition.Manual;
            disp.Top = 0;
            disp.Left = 0;
            PictureBox pcb = new PictureBox();
            pcb.Size = disp.ClientSize;
            pcb.Image = dst;
            disp.Controls.Add(pcb);

            disp.Show();
        }

        Bitmap original;
        Bitmap dst;
        Form disp = new Form();
    }
}